// *****************************************************************
// Filename:    IntensityGraph.h
// Copyright:   Kai Welke, Chair Prof. Dillmann (IAIM),
//              Institute for Computer Science and Engineering (CSE),
//              University of Karlsruhe. All rights reserved.
// Author:      Kai Welke
// Date:        12.06.2008
// *****************************************************************

#ifndef _INTENSITY_GRAPH_H_
#define _INTENSITY_GRAPH_H_

// *****************************************************************
// includes
// *****************************************************************
#include "SphericalGraph.h"
#include <string>

using namespace std;

// *****************************************************************
// namespaces
// *****************************************************************

// *****************************************************************
// structures
// *****************************************************************
// *****************************************************************
// definition of nodes
// *****************************************************************
class CIntensityNode : public CSGNode
{
public:
    CIntensityNode();
    CIntensityNode(TSphereCoord coord);
    ~CIntensityNode();

    void setIntensity(float fIntensity);
    float getIntensity();

    virtual CSGNode* clone()
    {
        CIntensityNode* copy = new CIntensityNode(m_Position);
        copy->setIndex(m_nIndex);
        copy->setIntensity(m_fIntensity);
        return copy;
    }

private:
    float   m_fIntensity;
};


// *****************************************************************
// definition of sensory egosphere
// *****************************************************************
class CIntensityGraph : public CSphericalGraph
{
public:
    CIntensityGraph();
    CIntensityGraph(const CIntensityGraph& prototype);
    //CIntensityGraph(int nNumberNodes);
    CIntensityGraph(string sFileName);
    ~CIntensityGraph();

    void reset();
    void set(float fIntensity);


    void graphToVec(std::vector<float>& vec);

    ///////////////////////////////
    // file io
    ///////////////////////////////
    virtual bool read(istream& infile);
    virtual bool readNode(istream& infile);
    virtual bool readEdge(istream& infile);

    virtual bool write(ostream& outfile);
    virtual bool writeNode(ostream& outfile, int n);
    virtual bool writeEdge(ostream& outfile, int e);


    // inherited
    virtual CIntensityNode* getNewNode()
    {
        return new CIntensityNode();
    }

    ///////////////////////////////
    // operators
    ///////////////////////////////
    CIntensityGraph& operator= (CIntensityGraph const& rhs);

};

#endif // _INTENSITY_GRAPH_H_ 
