/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::GamepadControlUnit
 * @author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_COMPONENT_RobotAPI_GamepadControlUnit_H
#define _ARMARX_COMPONENT_RobotAPI_GamepadControlUnit_H


#include <ArmarXCore/core/Component.h>

#include <RobotAPI/interface/units/GamepadUnit.h>

namespace armarx
{
    /**
     * @class GamepadControlUnitPropertyDefinitions
     * @brief
     */
    class GamepadControlUnitPropertyDefinitions:
        public armarx::ComponentPropertyDefinitions
    {
    public:
        GamepadControlUnitPropertyDefinitions(std::string prefix):
            armarx::ComponentPropertyDefinitions(prefix)
        {
            //defineRequiredProperty<std::string>("PropertyName", "Description");
            //defineOptionalProperty<std::string>("PropertyName", "DefaultValue", "Description");
            defineOptionalProperty<std::string>("PlatformUnitName", "Armar6IcePlatformUnit", "Name of the platform unit to use");
            defineOptionalProperty<std::string>("GamepadTopicName", "GamepadValues", "Name of the Gamepad Topic");
            defineOptionalProperty<float>("ScaleX", 1000.0f, "scaling factor in mm per second");
            defineOptionalProperty<float>("ScaleY", 1000.0f, "scaling factor in mm per second");
            defineOptionalProperty<float>("ScaleAngle", 1.0f, "scaling factor in radian per second");
        }
    };

    /**
     * @defgroup Component-GamepadControlUnit GamepadControlUnit
     * @ingroup RobotAPI-Components
     * A description of the component GamepadControlUnit.
     *
     * @class GamepadControlUnit
     * @ingroup Component-GamepadControlUnit
     * @brief Brief description of class GamepadControlUnit.
     *
     * Detailed description of class GamepadControlUnit.
     */
    class GamepadControlUnit :
        virtual public armarx::Component,
        virtual public GamepadUnitListener
    {
    public:
        /**
         * @see armarx::ManagedIceObject::getDefaultName()
         */
        virtual std::string getDefaultName() const
        {
            return "GamepadControlUnit";
        }

    protected:
        /**
         * @see armarx::ManagedIceObject::onInitComponent()
         */
        virtual void onInitComponent();

        /**
         * @see armarx::ManagedIceObject::onConnectComponent()
         */
        virtual void onConnectComponent();

        /**
         * @see armarx::ManagedIceObject::onDisconnectComponent()
         */
        virtual void onDisconnectComponent();

        /**
         * @see armarx::ManagedIceObject::onExitComponent()
         */
        virtual void onExitComponent();

        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        virtual armarx::PropertyDefinitionsPtr createPropertyDefinitions();

    private:
        PlatformUnitInterfacePrx platformUnitPrx;
        float scaleX;
        float scaleY;
        float scaleRotation;

    public:
        void reportGamepadState(const std::string& device, const std::string& name, const GamepadData& data,
                                const TimestampBasePtr& timestamp, const Ice::Current& c) override;

    };
}

#endif
