/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::ActuatorVelocityTarget
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_ControlTarget1DoFActuator_H
#define _ARMARX_LIB_RobotAPI_ControlTarget1DoFActuator_H

#include "ControlTargetBase.h"
#include <ArmarXCore/observers/variant/Variant.h>

namespace armarx
{
#define make_ControlTarget1DoFActuator(name,varname,cmode)                                                          \
    class name: public ControlTargetBase                                                                            \
    {                                                                                                               \
    public:                                                                                                         \
        float varname = ControllerConstants::ValueNotSetNaN;                                                        \
        virtual const std::string& getControlMode() const override                                                  \
        {                                                                                                           \
            return cmode;                                                                                           \
        }                                                                                                           \
        virtual void reset() override                                                                               \
        {                                                                                                           \
            varname = ControllerConstants::ValueNotSetNaN;                                                          \
        }                                                                                                           \
        virtual bool isValid() const override                                                                       \
        {                                                                                                           \
            return std::isfinite(varname);                                                                          \
        }                                                                                                           \
        virtual std::map<std::string, VariantBasePtr> toVariants(const IceUtil::Time& timestamp) const override     \
        {                                                                                                           \
            return {{#varname, {new TimedVariant{varname,timestamp}}}};                                             \
        }                                                                                                           \
        DETAIL_ControlTargetBase_DEFAULT_METHOD_IMPLEMENTATION                                                      \
    }

    make_ControlTarget1DoFActuator(ControlTarget1DoFActuatorPosition, position, ControlModes::Position1DoF);
    make_ControlTarget1DoFActuator(ControlTarget1DoFActuatorVelocity, velocity, ControlModes::Velocity1DoF);
    make_ControlTarget1DoFActuator(ControlTarget1DoFActuatorTorque  , torque  , ControlModes::Torque1DoF);

#undef make_ControlTarget1DoFActuator
}

#endif
