/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_SensorDevice_H
#define _ARMARX_LIB_RobotAPI_SensorDevice_H

#include"DeviceBase.h"
#include "../SensorValues/SensorValueBase.h"
#include "../util.h"

#include <IceUtil/Time.h>

namespace armarx
{
    TYPEDEF_PTRS_SHARED(SensorDevice);

    namespace SensorDeviceTags
    {
        using namespace DeviceTags;
    }

    class SensorDevice: public virtual DeviceBase
    {
    public:
        /// @brief A static const nullptr in case a const ref to a nullptr needs to be returned
        static const SensorDevicePtr NullPtr;
        /// @brief Create a SensorDevice with the given name
        SensorDevice(const std::string& name): DeviceBase {name} {}
        /// @return The SensorDevice's sensor value
        virtual const SensorValueBase* getSensorValue() const = 0;

        std::string getSensorValueType(bool withoutNamespaceSpecifier = false) const
        {
            return getSensorValue()->getSensorValueType(withoutNamespaceSpecifier);
        }

        virtual void rtReadSensorValues(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration) {}
    };
}

#endif
