/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Raphael ( raphael dot grimm at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#ifndef _ARMARX_LIB_RobotAPI_SensorValue1DoFActuator_H
#define _ARMARX_LIB_RobotAPI_SensorValue1DoFActuator_H

#include "SensorValueBase.h"

#include <ArmarXCore/core/util/algorithm.h>

namespace armarx
{
    class SensorValue1DoFActuatorPosition : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float position;
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return {{"position", {new TimedVariant{position, timestamp}}}};
        }
    };
    class SensorValue1DoFActuatorVelocity : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float velocity;
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return {{"velocity", {new TimedVariant{velocity, timestamp}}}};
        }
    };
    class SensorValue1DoFActuatorAcceleration : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float acceleration;
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return {{"acceleration", {new TimedVariant{acceleration, timestamp}}}};
        }
    };
    class SensorValue1DoFActuatorTorque : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float torque;
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return {{"torque", {new TimedVariant{torque, timestamp}}}};
        }
    };


    class SensorValue1DoFActuator :
        virtual public SensorValue1DoFActuatorPosition,
        virtual public SensorValue1DoFActuatorVelocity,
        virtual public SensorValue1DoFActuatorAcceleration,
        virtual public SensorValue1DoFActuatorTorque
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            auto map = SensorValue1DoFActuatorPosition::toVariants(timestamp);
            mergeMaps(map, SensorValue1DoFActuatorVelocity::toVariants(timestamp), MergeMapsMode::OverrideNoValues);
            mergeMaps(map, SensorValue1DoFActuatorAcceleration::toVariants(timestamp), MergeMapsMode::OverrideNoValues);
            mergeMaps(map, SensorValue1DoFActuatorTorque::toVariants(timestamp), MergeMapsMode::OverrideNoValues);
            return map;
        }
    };


    class SensorValue1DoFActuatorCurrent : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float motorCurrent;
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return {{"motorCurrent", {new TimedVariant{motorCurrent, timestamp}}}};
        }
    };
    class SensorValue1DoFActuatorMotorTemperature : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float motorTemperature;
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return {{"motorTemperature", {new TimedVariant{motorTemperature, timestamp}}}};
        }
    };

    class SensorValue1DoFRealActuator :
        virtual public SensorValue1DoFActuator,
        virtual public SensorValue1DoFActuatorCurrent,
        virtual public SensorValue1DoFActuatorMotorTemperature
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            auto map = SensorValue1DoFActuator::toVariants(timestamp);
            mergeMaps(map, SensorValue1DoFActuatorCurrent::toVariants(timestamp), MergeMapsMode::OverrideNoValues);
            mergeMaps(map, SensorValue1DoFActuatorMotorTemperature::toVariants(timestamp), MergeMapsMode::OverrideNoValues);
            return map;
        }
    };
}
#endif
