/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Raphael ( raphael dot grimm at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#ifndef _ARMARX_LIB_RobotAPI_SensorValueIMU_H
#define _ARMARX_LIB_RobotAPI_SensorValueIMU_H

#include <Eigen/Core>
#include <Eigen/Geometry>

#include "SensorValueBase.h"

#include <ArmarXCore/core/util/algorithm.h>
#include <RobotAPI/libraries/core/Pose.h>

namespace armarx
{
    class SensorValueIMU : public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        Eigen::Vector3f linearVelocity;
        Eigen::Vector3f angularVelocity;
        Eigen::Vector3f linearAcceleration;
        Eigen::Quaternionf orientation;
        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return
            {
                {"linearVelocity",      {new TimedVariant{Vector3{linearVelocity},timestamp}}},
                {"angularVelocity",     {new TimedVariant{Vector3{angularVelocity},timestamp}}},
                {"linearAcceleration",  {new TimedVariant{Vector3{linearAcceleration},timestamp}}},
                {"orientation",         {new TimedVariant{Quaternion{orientation},timestamp}}},
            };
        }
    };
}
#endif
