/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Raphael ( raphael dot grimm at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#ifndef _ARMARX_LIB_RobotAPI_SensorValueRTThreadTimings_H
#define _ARMARX_LIB_RobotAPI_SensorValueRTThreadTimings_H

#include "SensorValueBase.h"

#include <chrono>

#include <ArmarXCore/observers/variant/TimestampVariant.h>

namespace armarx
{
    class SensorValueRTThreadTimings : public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION

        std::chrono::microseconds rtSwitchControllerSetupDuration;
        std::chrono::microseconds rtSwitchControllerSetupRoundTripTime;

        std::chrono::microseconds rtRunLVL1ControllersDuration;
        std::chrono::microseconds rtRunLVL1ControllersRoundTripTime;

        std::chrono::microseconds rtHandleInvalidTargetsDuration;
        std::chrono::microseconds rtHandleInvalidTargetsRoundTripTime;

        std::chrono::microseconds rtRunLVL0ControllersDuration;
        std::chrono::microseconds rtRunLVL0ControllersRoundTripTime;

        std::chrono::microseconds rtBusSendReceiveDuration;
        std::chrono::microseconds rtBusSendReceiveRoundTripTime;

        std::chrono::microseconds rtReadSensorDeviceValuesDuration;
        std::chrono::microseconds rtReadSensorDeviceValuesRoundTripTime;

        std::chrono::microseconds rtUpdateSensorAndControlBufferDuration;
        std::chrono::microseconds rtUpdateSensorAndControlBufferRoundTripTime;

        std::chrono::microseconds rtResetAllTargetsDuration;
        std::chrono::microseconds rtResetAllTargetsRoundTripTime;

        std::chrono::microseconds rtLoopDuration;
        std::chrono::microseconds rtLoopRoundTripTime;

        virtual std::map<std::string, VariantBasePtr>   toVariants(const IceUtil::Time& timestamp) const override
        {
            return
            {
                    {"rtSwitchControllerSetup.Duration",                new TimedVariant{TimestampVariant{rtSwitchControllerSetupDuration.count()            }, timestamp}},
                    {"rtSwitchControllerSetup.RoundTripTime",           new TimedVariant{TimestampVariant{rtSwitchControllerSetupRoundTripTime.count()       }, timestamp}},
                    {"rtRunLVL1Controllers.Duration",                   new TimedVariant{TimestampVariant{rtRunLVL1ControllersDuration.count()               }, timestamp}},
                    {"rtRunLVL1Controllers.RoundTripTime",              new TimedVariant{TimestampVariant{rtRunLVL1ControllersRoundTripTime.count()          }, timestamp}},
                    {"rtHandleInvalidTargets.Duration",                 new TimedVariant{TimestampVariant{rtHandleInvalidTargetsDuration.count()             }, timestamp}},
                    {"rtHandleInvalidTargets.RoundTripTime",            new TimedVariant{TimestampVariant{rtHandleInvalidTargetsRoundTripTime.count()        }, timestamp}},
                    {"rtRunLVL0Controllers.Duration",                   new TimedVariant{TimestampVariant{rtRunLVL0ControllersDuration.count()               }, timestamp}},
                    {"rtRunLVL0Controllers.RoundTripTime",              new TimedVariant{TimestampVariant{rtRunLVL0ControllersRoundTripTime.count()          }, timestamp}},
                    {"rtBusSendReceive.Duration",                       new TimedVariant{TimestampVariant{rtBusSendReceiveDuration.count()                   }, timestamp}},
                    {"rtBusSendReceive.RoundTripTime",                  new TimedVariant{TimestampVariant{rtBusSendReceiveRoundTripTime.count()              }, timestamp}},
                    {"rtReadSensorDeviceValues.Duration",               new TimedVariant{TimestampVariant{rtReadSensorDeviceValuesDuration.count()           }, timestamp}},
                    {"rtReadSensorDeviceValues.RoundTripTime",          new TimedVariant{TimestampVariant{rtReadSensorDeviceValuesRoundTripTime.count()      }, timestamp}},
                    {"rtUpdateSensorAndControlBuffer.Duration",         new TimedVariant{TimestampVariant{rtUpdateSensorAndControlBufferDuration.count()     }, timestamp}},
                    {"rtUpdateSensorAndControlBuffer.RoundTripTime",    new TimedVariant{TimestampVariant{rtUpdateSensorAndControlBufferRoundTripTime.count()}, timestamp}},
                    {"rtResetAllTargets.Duration",                      new TimedVariant{TimestampVariant{rtResetAllTargetsDuration.count()                  }, timestamp}},
                    {"rtResetAllTargets.RoundTripTime",                 new TimedVariant{TimestampVariant{rtResetAllTargetsRoundTripTime.count()             }, timestamp}},
                    {"rtLoop.Duration",                                 new TimedVariant{TimestampVariant{rtLoopDuration.count()                             }, timestamp}},
                    {"rtLoop.RoundTripTime",                            new TimedVariant{TimestampVariant{rtLoopRoundTripTime.count()                        }, timestamp}},
            };
        }
    };
}
#endif
