/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::RobotUnitPluginWidgetController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_RobotAPI_RobotUnitPlugin_WidgetController_H
#define _ARMARX_RobotAPI_RobotUnitPlugin_WidgetController_H

#include "ui_RobotUnitPluginWidget.h"

#include <mutex>
#include <map>
#include <string>
#include <chrono>

#include <QWidget>
#include <QPushButton>
#include <QTreeWidget>
#include <QTreeWidgetItem>

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>

#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <ArmarXGui/libraries/SimpleConfigDialog/SimpleConfigDialog.h>

#include <RobotAPI/interface/units/RobotUnit/RobotUnitInterface.h>
#include <RobotAPI/libraries/RobotAPIVariantWidget/RobotAPIVariantWidget.h>

#include "RobotUnitLVL1ControllerClassesWidget.h"
#include "RobotUnitLVL1ControllersWidget.h"
#include "RobotUnitControlDevicesWidget.h"
#include "RobotUnitSensorDevicesWidget.h"

namespace armarx
{
    /**
    \page ArmarXGui-GuiPlugins-RobotUnitPlugin RobotUnitPlugin
    \brief The RobotUnitPlugin allows visualizing ...

    \image html RobotUnitPlugin.png
    The user can

    API Documentation \ref RobotUnitPluginWidgetController

    \see RobotUnitPluginGuiPlugin
    */

    /**
     * \class RobotUnitPluginWidgetController
     * \brief RobotUnitPluginWidgetController brief one line description
     *
     * Detailed description
     */
    class ARMARXCOMPONENT_IMPORT_EXPORT RobotUnitPluginWidgetController:
        public ArmarXComponentWidgetController,
        public RobotUnitListener
    {
        Q_OBJECT
    public:
        /// @brief Controller Constructor
        explicit RobotUnitPluginWidgetController();

        /// Controller destructor
        virtual ~RobotUnitPluginWidgetController();

        /// @see ArmarXWidgetController::loadSettings()
        virtual void loadSettings(QSettings* settings) override;

        /// @see ArmarXWidgetController::saveSettings()
        virtual void saveSettings(QSettings* settings) override;

        /// @brief Returns the Widget name displayed in the ArmarXGui to create an instance of this class.
        virtual QString getWidgetName() const override
        {
            return "RobotControl.RobotUnitGUI";
        }

        /// @see armarx::Component::onInitComponent()
        virtual void onInitComponent() override;

        /// @see armarx::Component::onConnectComponent()
        virtual void onConnectComponent() override;

        virtual void onDisconnectComponent() override;
        /**
         * @param parent The dialog's parent.
         * @return The plugin's config dialog.
         */
        QPointer<QDialog> getConfigDialog(QWidget* parent);

        /// @brief Callback called when the config dialog is closed.
        virtual void configured() override;

        virtual QIcon getWidgetIcon() const override
        {
            return QIcon(":/robotunitplugin/icons/robot_unit_icon.svg");
        }
        // RobotUnitListener interface
    public:
        virtual void lVl1ControllerStatusChanged(const LVL1ControllerStatus& status, const Ice::Current&) override;
        virtual void controlDeviceStatusChanged(const ControlDeviceStatus& status, const Ice::Current&) override;
        virtual void sensorDeviceStatusChanged(const SensorDeviceStatus& status, const Ice::Current&) override;
        virtual void lvl1ControllerClassAdded(const std::string& name, const Ice::Current&) override;
        virtual void lvl1ControllerCreated(const std::string& name, const Ice::Current&) override;
        virtual void lvl1ControllerDeleted(const std::string& name, const Ice::Current&) override;

    private slots:
        void refreshLVL1ControllersClicked();
        void refreshLVL1ControllerClassesClicked();
        void refreshControlDevicesClicked();
        void refreshSensorDevicesClicked();
        void changeVisibleCount();
        void loadLibClicked();
        void packageEditChanged();

    private:
        /// @brief Widget Form
        Ui::RobotUnitPluginWidget widget;
        /// @brief The plugin's config dialog.
        QPointer<SimpleConfigDialog> dialog;

        std::string robotUnitProxyName;
        RobotUnitInterfacePrx robotUnitPrx;
        std::string listenerTopicName;

        RobotUnitLVL1ControllerClassesWidget* lvl1ControllerClasses;
        RobotUnitLVL1ControllersWidget* lvl1Controllers;
        RobotUnitControlDevicesWidget* controlDevices;
        RobotUnitSensorDevicesWidget* sensorDevices;
        QSpacerItem* spacer;
        bool spacerAdded = false;

        std::tuple<QWidget*, QCheckBox*, QPushButton*> makeHeaderItem(const QString& name);
        //selecting libs
        enum class SelectLibsMode
        {
            LineEdit,
            ComboBox
        };
        SelectLibsMode selectLibMode;
        std::map<std::string, std::string> libShortNameToFileName;
    };
}

#endif
