/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * \package    RobotAPI::gui-plugins::RobotUnitPlugin
 * \author     Raphael ( ufdrv at student dot kit dot edu )
 * \date       2017
 * \copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_RobotAPI_RobotUnitPlugin_RobotUnitSensorDevicesWidget_H
#define _ARMARX_RobotAPI_RobotUnitPlugin_RobotUnitSensorDevicesWidget_H

#include <mutex>

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QPushButton>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QTreeWidgetItem>

#include <ArmarXGui/libraries/WidgetDescription/WidgetDescription.h>
#include <ArmarXGui/libraries/VariantWidget/VariantWidget.h>
#include <RobotAPI/interface/units/RobotUnit/RobotUnitInterface.h>

namespace Ui
{
    class RobotUnitSensorDevicesWidget;
}

namespace armarx
{
    class RobotUnitSensorDevicesWidgetEntry;

    class RobotUnitSensorDevicesWidget : public QWidget
    {
        Q_OBJECT

    public:
        explicit RobotUnitSensorDevicesWidget(QWidget* parent = 0);
        ~RobotUnitSensorDevicesWidget();

        void sensorDeviceStatusChanged(const SensorDeviceStatus& status);
    public slots:
        void reset(RobotUnitInterfacePrx ru);
    private slots:
        void sensorDeviceStatusChanged();
    private:
        void add(const SensorDeviceDescription& desc);

        std::map<std::string, RobotUnitSensorDevicesWidgetEntry*> entries;
        RobotUnitInterfacePrx robotUnit;
        mutable std::recursive_mutex mutex;
        Ui::RobotUnitSensorDevicesWidget* ui;

        mutable std::recursive_mutex statusUpdatesMutex;
        std::map<std::string, SensorDeviceStatus> statusUpdates;

    public:
        static constexpr int idxName    = 0;
        static constexpr int idxTags    = 1;
        static constexpr int idxValType = 2;
        static constexpr int idxValue   = 3;
    };

    class RobotUnitSensorDevicesWidgetEntry : QObject
    {

        Q_OBJECT
    public:
        RobotUnitSensorDevicesWidgetEntry(
            RobotUnitSensorDevicesWidget& parent,
            QTreeWidget& treeWidget,
            const SensorDeviceDescription& desc
        );

        void update(const SensorDeviceStatus& status);

        bool matchName(const QString& name);
        bool matchTag(const QString& tag);
        bool matchValueType(const QString& type);

    public slots:
        void setVisible(bool vis);

    private slots:
        void hideTagList(bool hide);

    private:
        QTreeWidgetItem* header;
        VariantWidget* value;
        std::vector<QTreeWidgetItem*> tags;
    };
}

#endif
