/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::LVL1ControllerInterface
 * @author     Raphael Grimm ( raphael dot grimm at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_ROBOTAPI_LVL1ControllerInterface_SLICE_
#define _ARMARX_ROBOTAPI_LVL1ControllerInterface_SLICE_

#include <ArmarXCore/interface/core/BasicTypes.ice>
#include <ArmarXCore/interface/core/UserException.ice>
#include <ArmarXGui/interface/WidgetDescription.ice>

module armarx
{
    interface LVL1ControllerInterface;
    interface RobotUnitInterface;

    class LVL1ControllerConfig{};

    struct LVL1ControllerDescription
    {
        string instanceName;
        string className;
        LVL1ControllerInterface* controller;
        StringStringDictionary controlModeAssignment;
        bool deletable;
    };
    sequence<LVL1ControllerDescription> LVL1ControllerDescriptionSeq;

    struct LVL1ControllerStatus
    {
        string instanceName;
        bool active = false;
        bool requested = false;
        bool error = false;
        long timestampUSec = 0;
    };
    sequence<LVL1ControllerStatus> LVL1ControllerStatusSeq;

    struct LVL1ControllerDescriptionWithStatus
    {
        LVL1ControllerStatus status;
        LVL1ControllerDescription description;
    };
    sequence<LVL1ControllerDescriptionWithStatus> LVL1ControllerDescriptionWithStatusSeq;

    interface LVL1ControllerInterface
    {
        ["cpp:const"] idempotent string getClassName();
        ["cpp:const"] idempotent string getInstanceName();
        ["cpp:const"] idempotent StringStringDictionary getControlDeviceUsedControlModeMap();
        ["cpp:const"] idempotent bool isControllerActive();
        ["cpp:const"] idempotent bool isControllerRequested();
        ["cpp:const"] idempotent bool isDeletable();
        ["cpp:const"] idempotent bool hasControllerError();
        ["cpp:const"] idempotent LVL1ControllerStatus getControllerStatus();
        ["cpp:const"] idempotent LVL1ControllerDescription getControllerDescription();
        ["cpp:const"] idempotent LVL1ControllerDescriptionWithStatus getControllerDescriptionWithStatus();
        ["cpp:const"] idempotent RobotUnitInterface* getRobotUnit();

        ["cpp:const"] void activateController();
        ["cpp:const"] void deactivateController();
        ["cpp:const"] void deleteController() throws LogicError;

        ["cpp:const"] idempotent WidgetDescription::StringWidgetDictionary getFunctionDescriptions();
        void callDescribedFunction(string fuinctionName, StringVariantBaseMap values) throws InvalidArgumentException;
    };

    dictionary<string, LVL1ControllerInterface*> StringLVL1ControllerPrxDictionary;
};
#endif
